@@ -9,7 +9,7 @@ use std::collections::HashMap;
 use std::os::unix::io::RawFd;
 use std::sync::{Arc, Mutex};
 
-use crate::common::MESSAGE_TYPE_RESPONSE;
+use crate::common::{client_connect, MESSAGE_TYPE_RESPONSE};
 use crate::error::{Error, Result};
 use crate::ttrpc::{Code, Request, Response};
 
@@ -35,6 +35,11 @@ pub struct Client {
 }
 
 impl Client {
+    pub fn connect(path: &str) -> Result<Client> {
+        let fd = unsafe { client_connect(path)? };
+        Ok(Self::new(fd))
+    }
+
     /// Initialize a new [`Client`].
     pub fn new(fd: RawFd) -> Client {
         let stream = utils::new_unix_stream_from_raw_fd(fd);
@@ -156,10 +161,19 @@ impl Client {
             .await
             .map_err(|e| Error::Others(format!("Send packet to sender error {:?}", e)))?;
 
-        let result = rx
-            .recv()
+        let result = if req.timeout_nano == 0 {
+            rx.recv()
+                .await
+                .ok_or_else(|| Error::Others("Receive packet from receiver error".to_string()))?
+        } else {
+            tokio::time::timeout(
+                std::time::Duration::from_nanos(req.timeout_nano as u64),
+                rx.recv(),
+            )
             .await
-            .ok_or_else(|| Error::Others("Recive packet from recver error".to_string()))?;
+            .map_err(|e| Error::Others(format!("Receive packet timeout {:?}", e)))?
+            .ok_or_else(|| Error::Others("Receive packet from receiver error".to_string()))?
+        };
 
         let buf = result?;
         let mut s = CodedInputStream::from_bytes(&buf);
