@@ -33,6 +33,8 @@ use tokio::{
     sync::watch,
     time::timeout,
 };
+
+#[cfg(target_os = "linux")]
 use tokio_vsock::VsockListener;
 
 /// A ttrpc Server (async).
@@ -83,6 +85,7 @@ impl Server {
         self
     }
 
+    #[cfg(target_os = "linux")]
     pub fn set_domain_vsock(mut self) -> Self {
         self.domain = Some(Domain::Vsock);
         self
@@ -129,21 +132,20 @@ impl Server {
 
                 let incoming = UnixIncoming::new(unix_listener);
 
-                self.do_start(listenfd, incoming).await
+                self.do_start(incoming).await
             }
+            #[cfg(target_os = "linux")]
             Some(Domain::Vsock) => {
-                let incoming;
-                unsafe {
-                    incoming = VsockListener::from_raw_fd(listenfd).incoming();
-                }
-
-                self.do_start(listenfd, incoming).await
+                let incoming = unsafe { VsockListener::from_raw_fd(listenfd).incoming() };
+                self.do_start(incoming).await
             }
-            _ => Err(Error::Others("Domain is not set".to_string())),
+            _ => Err(Error::Others(
+                "Domain is not set or not supported".to_string(),
+            )),
         }
     }
 
-    pub async fn do_start<I, S>(&mut self, listenfd: RawFd, mut incoming: I) -> Result<()>
+    pub async fn do_start<I, S>(&mut self, mut incoming: I) -> Result<()>
     where
         I: Stream<Item = std::io::Result<S>> + Unpin + Send + 'static + AsRawFd,
         S: AsyncRead + AsyncWrite + AsRawFd + Send + 'static,
@@ -168,14 +170,9 @@ impl Server {
                             match conn {
                                 Ok(stream) => {
                                     let fd = stream.as_raw_fd();
-                                    if let Err(e) = common::set_fd_close_exec(fd) {
-                                        error!("{:?}", e);
-                                        continue;
-                                    }
-
                                     // spawn a connection handler, would not block
                                     spawn_connection_handler(
-                                        listenfd,
+                                        fd,
                                         stream,
                                         methods.clone(),
                                         close_conn_rx.clone(),
@@ -240,7 +237,7 @@ impl Server {
 }
 
 async fn spawn_connection_handler<S>(
-    listenfd: RawFd,
+    fd: RawFd,
     stream: S,
     methods: Arc<HashMap<String, Box<dyn MethodHandler + Send + Sync>>>,
     mut close_conn_rx: watch::Receiver<i32>,
@@ -275,7 +272,7 @@ async fn spawn_connection_handler<S>(
                         Ok(message) => {
                             spawn(async move {
                                 select! {
-                                    _ = handle_request(tx, listenfd, methods, message) => {}
+                                    _ = handle_request(tx, fd, methods, message) => {}
                                     _ = client_disconnected_rx2.changed() => {}
                                 }
 
@@ -322,6 +319,7 @@ async fn do_handle_request(
         fd,
         mh: header,
         metadata: context::from_pb(&req.metadata),
+        timeout_nano: req.timeout_nano,
     };
 
     let get_unknown_status_and_log_err = |e| {
