@@ -14,10 +14,8 @@
 
 //! Sync server of ttrpc.
 
-use nix::fcntl::OFlag;
 use nix::sys::socket::{self, *};
-use nix::unistd::close;
-use nix::unistd::pipe2;
+use nix::unistd::*;
 use protobuf::{CodedInputStream, Message};
 use std::collections::HashMap;
 use std::os::unix::io::{AsRawFd, FromRawFd, RawFd};
@@ -27,6 +25,8 @@ use std::sync::{Arc, Mutex};
 use std::thread::JoinHandle;
 use std::{io, thread};
 
+#[cfg(not(target_os = "linux"))]
+use crate::common::set_fd_close_exec;
 use crate::common::{self, MESSAGE_TYPE_REQUEST};
 use crate::context;
 use crate::error::{get_status, Error, Result};
@@ -186,9 +186,8 @@ fn start_method_handler_thread(
             trace!("Got Message request {:?}", req);
 
             let path = format!("/{}/{}", req.service, req.method);
-            let method;
-            if let Some(x) = methods.get(&path) {
-                method = x;
+            let method = if let Some(x) = methods.get(&path) {
+                x
             } else {
                 let status = get_status(Code::INVALID_ARGUMENT, format!("{} does not exist", path));
                 let mut res = Response::new();
@@ -205,12 +204,13 @@ fn start_method_handler_thread(
                     break;
                 }
                 continue;
-            }
+            };
             let ctx = TtrpcContext {
                 fd,
                 mh,
                 res_tx: res_tx.clone(),
                 metadata: context::from_pb(&req.metadata),
+                timeout_nano: req.timeout_nano,
             };
             if let Err(x) = method.handler(ctx, req) {
                 debug!("method handle {} get error {:?}", path, x);
@@ -249,7 +249,7 @@ fn start_method_handler_threads(num: usize, ts: &ThreadS) {
 fn check_method_handler_threads(ts: &ThreadS) {
     let c = ts.wtc.load(Ordering::SeqCst);
     if c < ts.min {
-        start_method_handler_threads(ts.default - c, &ts);
+        start_method_handler_threads(ts.default - c, ts);
     }
 }
 
@@ -327,8 +327,19 @@ impl Server {
         }
 
         self.listener_quit_flag.store(false, Ordering::SeqCst);
-        let (rfd, wfd) = pipe2(OFlag::O_CLOEXEC).unwrap();
-        self.monitor_fd = (rfd, wfd);
+
+        #[cfg(target_os = "linux")]
+        let fds = pipe2(nix::fcntl::OFlag::O_CLOEXEC)?;
+
+        #[cfg(not(target_os = "linux"))]
+        let fds = {
+            let (rfd, wfd) = pipe()?;
+            set_fd_close_exec(rfd)?;
+            set_fd_close_exec(wfd)?;
+            (rfd, wfd)
+        };
+
+        self.monitor_fd = fds;
 
         let listener = self.listeners[0];
 
@@ -352,7 +363,10 @@ impl Server {
                                 .unwrap()
                                 .remove(&fd)
                                 .map(|mut cn| {
-                                    cn.handler.take().map(|handler| handler.join().unwrap())
+                                    cn.handler.take().map(|handler| {
+                                        handler.join().unwrap();
+                                        close(fd).unwrap();
+                                    })
                                 });
                         }
                         info!("reaper thread exited");
@@ -394,7 +408,7 @@ impl Server {
                         let pollers: &mut [libc::pollfd] = &mut pollers;
                         libc::poll(
                             pollers as *mut _ as *mut libc::pollfd,
-                            pollers.len() as u64,
+                            pollers.len() as _,
                             -1,
                         )
                     };
@@ -420,6 +434,7 @@ impl Server {
                         break;
                     }
 
+                    #[cfg(target_os = "linux")]
                     let fd = match accept4(listener, SockFlag::SOCK_CLOEXEC) {
                         Ok(fd) => fd,
                         Err(e) => {
@@ -427,6 +442,25 @@ impl Server {
                             break;
                         }
                     };
+
+                    // Non Linux platforms do not support accept4 with SOCK_CLOEXEC flag, so instead
+                    // use accept and call fcntl separately to set SOCK_CLOEXEC.
+                    // Because of this there is chance of the descriptor leak if fork + exec happens in between.
+                    #[cfg(not(target_os = "linux"))]
+                    let fd = match accept(listener) {
+                        Ok(fd) => {
+                            if let Err(err) = set_fd_close_exec(fd) {
+                                error!("fcntl failed after accept: {:?}", err);
+                                break;
+                            };
+                            fd
+                        }
+                        Err(e) => {
+                            error!("failed to accept error {:?}", e);
+                            break;
+                        }
+                    };
+
                     let methods = methods.clone();
                     let quit = Arc::new(AtomicBool::new(false));
                     let child_quit = quit.clone();
@@ -480,7 +514,8 @@ impl Server {
                             // drop the res_tx, thus the res_rx would get terminated notification.
                             drop(res_tx);
                             handler.join().unwrap_or(());
-                            close(fd).unwrap_or(());
+                            // client_handler should not close fd before exit
+                            // , which prevent fd reuse issue.
                             reaper_tx_child.send(fd).unwrap();
 
                             debug!("client thread quit");
